from time import sleep
from recipys.ConfigFile import ConfigFile
from recipys.constants import MIN_SECONDS_REQUEST as min_seconds


def wait_for_green_light() -> None:
    """
    Checks config file and waits a given timeout before exiting.
    This avoids overloading the server with requests in a short
    period of time (ethical scraping & avoiding blocks)
    """

    seconds_passed = ConfigFile().get_delta_last_request()

    if seconds_passed < min_seconds:
        sleep(min_seconds - seconds_passed)

    ConfigFile().update_time_last_request()



from dataclasses import dataclass

# from rich import print

from recipys.types import RecipeInformation


@dataclass
class ConsolePrinter:
    """Prints recipe information to console"""

    recipe: RecipeInformation

    def print_recipe(self) -> None:
        """Print recipe"""
        if self.recipe.error_message:
            print(self.recipe.error_message)
        else:
            print(self.recipe.title)
            print(self.recipe.ingredients)
            print(self.recipe.preparation)



from dataclasses import dataclass
from typing import Optional, List


@dataclass
class RecipeConstraints:
    meal: Optional[str] = None
    ingredients: Optional[List[str]] = None


@dataclass
class RecipeInformation:
    title: str = ""
    ingredients: str = ""
    preparation: str = ""
    error_message: Optional[str] = None


@dataclass
class FetchingError(Exception):
    message: str = "An error ocurred"



import sys
from recipys.ArgParser import ArgParser
from recipys.ConsolePrinter import ConsolePrinter
from recipys.RecipeFetcher import RecipeFetcher
from recipys.request_wait import wait_for_green_light


def main():
    """Main entry point of app"""

    wait_for_green_light()

    recipe_constraints = ArgParser(sys.argv).parse()

    recipe = RecipeFetcher(recipe_constraints).fetch()

    ConsolePrinter(recipe).print_recipe()


if __name__ == "__main__":
    main()  # pragma: no cover



from dataclasses import dataclass
from typing import Dict, List, Optional, Tuple

from bs4 import BeautifulSoup
import requests

from recipys.types import FetchingError


@dataclass
class HtmlSearchTarget:
    """Search target for HTML elements and their values"""

    name: str
    tag: Optional[str] = None
    att_name: Optional[str] = None
    att_value: Optional[str] = None
    target_element: Optional[str] = None  # If none, inner HTML is returned


@dataclass
class ScraperSearchTerms:
    """Search terms for scraper within HTML page"""

    target: HtmlSearchTarget
    return_multiple: bool = False  # If False, only first match is returned


@dataclass
class Scraper:
    """Scrapes content from website"""

    url: str
    search_terms: List[ScraperSearchTerms]

    def get(self) -> Dict[str, List[str]]:
        """Return found data as requested by search parameters"""
        self._http_response = requests.get(self.url)

        try:
            self._check_http_response_status()
        except ConnectionRefusedError:
            raise FetchingError(
                (
                    "HTTP request error. "
                    "Please check your internet connection and try again"
                )
            )

        self._html = self._http_response.text
        return self._parse()

    def _check_http_response_status(self) -> None:
        """Check response of http GET request
        - Raises: ConnectionRefusedError if request's status is invalid"""
        try:
            self._http_response.raise_for_status()
        except requests.exceptions.HTTPError:
            raise ConnectionRefusedError

    def _parse(self) -> Dict[str, List[str]]:
        """Return Dict with findings in HTML file according to search terms"""
        parsed_results: Dict[str, List[str]] = {}

        for search in self.search_terms:
            results: List[str] = []

            attributes = (
                {search.target.att_name: search.target.att_value}
                if (search.target.att_name and search.target.att_name)
                else None
            )

            soup = BeautifulSoup(self._html, "html.parser")
            tags = soup.find_all(
                name=search.target.tag,
                attrs=attributes,
            )

            for tag in tags:
                # Get only exact matches
                if attributes:
                    if not tag.get_attribute_list(
                        f"{search.target.att_name}"
                    ) == [f"{search.target.att_value}"]:
                        continue

                if search.target.target_element:
                    # Specific tag of element is searched
                    att_value = tag.attrs.get(
                        search.target.target_element, None
                    )
                    if att_value:
                        results.append(att_value)
                else:
                    # Inner HTML is searched
                    results.append(tag.text)

                # Break if we only need one result
                if not search.return_multiple:
                    break

            parsed_results.setdefault(search.target.name, results)

        return parsed_results



import time
import json
from io import TextIOWrapper
from typing import Dict


class ConfigFile:
    """Handles reading and writing of .json config file"""

    def __init__(self) -> None:
        """Initialize instance variables"""

        self.file_name: str = "config.json"
        self.user_agent: str = (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:89.0) "
            "Gecko/20100101 Firefox/89.0"
        )
        self.headers: Dict[str, str] = {
            "User-Agent": self.user_agent,
            "Accept": (
                "text/html,application/xhtml+xml,application/xml"
                ";q=0.9,image/webp,*/*;q=0.8"
            ),
            "Accept-Language": "en-US,en;q=0.5",
            "Accept-Encoding": "gzip, deflate, br",
            "Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "Referer": "https://www.duckduckgo.com/",
        }
        self.last_request: float = time.time()

    def get_delta_last_request(self) -> float:
        """Return seconds since last request (saved in config file)"""

        self._read_config_file()
        return time.time() - self.last_request

    def update_time_last_request(self) -> None:
        """Update timestamp of last request, both local and in config file"""
        self._create_config_file()

    def _read_config_file(self) -> None:
        """Read config file and update instance variables"""
        try:
            with open(self.file_name, "r") as file:
                self._update_from_file(file)
        except OSError:
            self._create_config_file()

    def _update_from_file(self, file: TextIOWrapper) -> None:
        """
        Update instance variables from file

        Args:
            - file: file handle of json file

        Raises:
            - OSError: if KeyError Exception ocurrs (accessing dict)
        """
        data: Dict[str, str] = json.load(file)
        try:
            self.headers = self._get_headers_from_json_file(data)
            self.user_agent = self.headers["User-Agent"]
            self.last_request = self._get_last_request_from_json_file(data)
        except KeyError:
            raise OSError

    def _create_config_file(self) -> None:
        """Create the config file anew using default values"""
        new_config_file = ConfigFile()
        self.headers = new_config_file.headers
        self.last_request = new_config_file.last_request - 100
        dict_for_json_file: Dict[str, str] = {
            "headers": str(self.headers),
            "last_request": str(self.last_request),
        }
        with open(self.file_name, "w") as file:
            json.dump(dict_for_json_file, file)

    def _get_headers_from_json_file(
        self, data: Dict[str, str]
    ) -> Dict[str, str]:
        """
        Return headers dictionary from json loaded data

        Args:
            - data: dictionary of loaded json data

        Raises:
            - KeyError: if json file is tampered
                (instance keys and dictionary are different)
        """
        headers: str = data["headers"].replace("'", '"')
        json_headers: Dict[str, str] = json.loads(headers)
        if json_headers.keys() != self.headers.keys():
            raise KeyError  # tampered json file -> reset it
        return json_headers

    def _get_last_request_from_json_file(self, data: Dict[str, str]) -> str:
        """
        Return last_request dictionary value from json loaded data

        Args:
            - data: dictionary of loaded json data

        Raises:
            - KeyError: if json file is tampered
                (instance keys and dictionary are different)
        """
        try:
            json_num = float(data.get("last_request"))
            own_num = float(self.last_request)
            if json_num == 0 or own_num < json_num:
                raise ValueError
            return json_num
        except ValueError:
            raise KeyError



from dataclasses import dataclass
import re
from time import sleep

from recipys.types import RecipeConstraints, RecipeInformation, FetchingError
from recipys.Scraper import (
    Scraper,
    ScraperSearchTerms,
    HtmlSearchTarget,
)
from recipys.constants import WAIT_BETWEEN_PINGS


@dataclass
class RecipeFetcher:
    """Fetches recipes from the web"""

    recipe_constraints: RecipeConstraints

    def fetch(self) -> RecipeInformation:
        """Fetches recipe according to user input"""
        try:
            self.recipe_url = self._scrape_recipe_url()
            sleep(WAIT_BETWEEN_PINGS)
            recipe = self._scrape_recipe()
            return recipe
        except FetchingError as e:
            return RecipeInformation(error_message=e.message)

    def _get_url_recipe(self) -> str:
        """Get URL for HTTP GET request"""
        url_base: str = "https://www.recipe-free.com/recipe/"
        url_suffix: str = "/1/search"
        ings = (
            self.recipe_constraints.ingredients
            if self.recipe_constraints.ingredients
            else [None]
        )
        query = [self.recipe_constraints.meal] + ings
        clean_query = [e for e in query if e]

        if clean_query:
            return url_base + "-".join(clean_query) + url_suffix
        else:
            return "https://www.recipe-free.com/best-recipes/1"

    def _scrape_recipe_url(self) -> str:
        """Scrape url of recipe according to search constraints"""
        scraper = Scraper(
            url=self._get_url_recipe(),
            search_terms=[
                ScraperSearchTerms(
                    target=HtmlSearchTarget(
                        name="Recipe",
                        tag="a",
                        att_name="class",
                        att_value="day",
                        target_element="href",
                    )
                )
            ],
        )

        recipe_url = scraper.get().get("Recipe")[0]
        if not recipe_url:
            raise FetchingError(
                "No recipe found with your criteria. "
                "Maybe try removing or changing your filters to broaden your search?"
            )

        return recipe_url

    def _scrape_recipe(self) -> RecipeInformation:
        """Scrape recipe information from its URL"""
        scraper = self._setup_scraper_recipe()
        results = scraper.get()

        try:
            recipe_title = self._beautify(results["Title"][0])
            recipe_ingredients = self._beautify(
                results["Ingredients & Preparation"][0]
            )
            recipe_preparation = self._beautify(
                results["Ingredients & Preparation"][1]
            )
        except KeyError or IndexError:
            raise FetchingError("Recipe format incorrect. Please try again")
        else:
            return RecipeInformation(
                title=recipe_title,
                ingredients=recipe_ingredients,
                preparation=recipe_preparation,
            )

    def _setup_scraper_recipe(self) -> Scraper:
        """Setup Scraper object to be used scraping recipe from its page"""
        target_title = HtmlSearchTarget(
            name="Title",
            tag="h1",
            att_name="class",
            att_value="red",
        )
        target_ings_and_prep = HtmlSearchTarget(
            name="Ingredients & Preparation",
            tag="p",
            att_name="style",
            att_value="padding-left: 30px",
        )

        search_terms_title = ScraperSearchTerms(target=target_title)
        search_terms_ings_and_prep = ScraperSearchTerms(
            target=target_ings_and_prep, return_multiple=True
        )

        scraper = Scraper(
            url=self.recipe_url,
            search_terms=[
                search_terms_title,
                search_terms_ings_and_prep,
            ],
        )

        return scraper

    def _beautify(self, text: str):
        """Return beautified text from parsed html"""

        # Remove multiple spaces and leading and trailing spaces
        return re.sub(" +", " ", text).strip()



"""
Global constants of application
"""

# Real time delay between two executions of application (ethical scraping)
MIN_SECONDS_REQUEST: float = 10.0


# Seconds to wait between requests to same endpoint (ethical scraping)
WAIT_BETWEEN_PINGS: int = 1



from dataclasses import dataclass
import re
from typing import List, Tuple, Optional

from recipys.types import RecipeConstraints


@dataclass
class ArgParser:
    """Command line arguments parser"""

    args: List[str]
    accepted_meals: Tuple[str] = (
        "breakfast",
        "lunch",
        "dinner",
        "dessert",
    )

    def parse(self) -> RecipeConstraints:
        """Parses command line arguments"""
        self.meal = self._get_meal()
        self.ingredients = self._get_ingredients()
        return RecipeConstraints(self.meal, self.ingredients)

    def _get_meal(self) -> Optional[str]:
        """Gets meal type from command line argument"""
        if len(self.args) > 1:
            for accepted_meal in self.accepted_meals:
                if self.args[1].lower() == accepted_meal:
                    return accepted_meal
        return None

    def _get_ingredients(self) -> Optional[List[str]]:
        """Gets meal type from command line argument"""
        args = [a.lower() for a in self.args]
        if "with" not in args:
            return None

        start: int = 2 if self.meal else 1

        if "with" != args[start]:
            return None

        ingredients: List[str] = []
        for _, arg in enumerate(args[start + 1 : :]):
            m = re.findall(r"[a-zA-Z]", arg)
            word = "".join(m)
            if word:
                ingredients.append(word)

        if not ingredients:
            return None

        return ingredients



import setuptools
import re
import ast


with open("README.md", "r") as fh:
    long_description = fh.read()


with open("recipys/__init__.py", "rb") as f:
    _app_name_re = re.compile(r"__app_name__\s+=\s+(.*)")
    app_name = str(
        ast.literal_eval(
            _app_name_re.search(f.read().decode("utf-8")).group(1)
        )
    )

with open("recipys/__init__.py", "rb") as f:
    _version_re = re.compile(r"__version__\s+=\s+(.*)")
    version = str(
        ast.literal_eval(_version_re.search(f.read().decode("utf-8")).group(1))
    )


setuptools.setup(
    name=app_name,
    version=version,
    author="Germán Mené Santa Olaya",
    author_email="german.mene@gmail.com",
    description=(
        "Get recipes instantly with this CLI tool. "
        "Choose specific meals or ingredients to cater to your appetite!"
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=f"https://github.com/gmso/{app_name}",
    packages=setuptools.find_packages(
        include=[
            f"{app_name}*",
        ]
    ),
    python_requires=">3.8.0",
    install_requires=[
        "beautifulsoup4",
        "requests" "rich",
    ],
    setup_requires=["pytest-runner", "flake8", "black"],
    tests_require=["pytest", "pytest-cov"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Environment :: Console",
    ],
    entry_points={"console_scripts": [f"{app_name} = {app_name}.App:main"]},
)
